package co.id.keda87.androsqlite.database;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import co.id.keda87.androsqlite.model.Contact;

import java.util.ArrayList;
import java.util.List;

/**
 * Created with IntelliJ IDEA.
 * User: Keda87
 * Date: 11/2/13
 * Time: 10:46 PM
 */
public class DatabaseHandler extends SQLiteOpenHelper {

    private static final int DATABASE_VERSION = 1;
    private static final String DATABASE_NAME = "contactManager";
    private static final String TABLE_CONTACTS = "contacts";
    private static final String COLUMN_ID = "id";
    private static final String COLUMN_NAME = "name";
    private static final String COLUMN_PHONE = "phone_number";

    public DatabaseHandler(Context context) {
        super(context, DATABASE_NAME, null, DATABASE_VERSION);
    }

    /**
     * Create Table Query
     * @param db
     */
    @Override
    public void onCreate(SQLiteDatabase db) {
        String CREATE_CONTACTS_TABLE = "CREATE TABLE " + TABLE_CONTACTS +
                                        "("+ COLUMN_ID + " INTEGER PRIMARY KEY,"
                                           + COLUMN_NAME + " TEXT,"
                                           + COLUMN_PHONE + " TEXT" + ")";
        db.execSQL(CREATE_CONTACTS_TABLE);
    }

    /**
     * Drop table if SQLite table above already exist
     * @param db
     * @param oldVersion
     * @param newVersion
     */
    @Override
    public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
        //drop older table if exists
        db.execSQL("DROP TABLE IF EXISTS " + TABLE_CONTACTS);

        //recreate table
        onCreate(db);
    }

    /**
     * Insert new row
     * @param contact
     */
    public void addContact(Contact contact) {
        SQLiteDatabase db = this.getWritableDatabase();

        ContentValues values = new ContentValues();
        values.put(COLUMN_NAME, contact._name);
        values.put(COLUMN_PHONE, contact._phoneNumber);

        //insert into database
        db.insert(TABLE_CONTACTS, null, values);
        db.close();
    }

    /**
     * Get contact by id
     * @param id
     * @return Contact
     */
    public Contact getContact(int id){
        SQLiteDatabase db = this.getReadableDatabase();

        Cursor cursor = db.query(
                TABLE_CONTACTS,
                new String[]{COLUMN_ID, COLUMN_NAME, COLUMN_PHONE}, COLUMN_ID + "=?",
                new String[]{String.valueOf(id)}, null, null, null, null
        );

        if(cursor != null) {
            cursor.moveToFirst();
        }

        Contact contact = new Contact(
                Integer.parseInt(cursor.getString(0)),
                cursor.getString(1),
                cursor.getString(2)
        );
        return contact;
    }

    /**
     * Populate all contacts from SQLite
     * @return List
     */
    public List<Contact> getAllContacts(){
        List<Contact> contactList = new ArrayList<>();
        //select query
        String selectQuery = "SELECT * FROM " + TABLE_CONTACTS;

        SQLiteDatabase db = this.getWritableDatabase();
        Cursor cursor = db.rawQuery(selectQuery, null);

        if(cursor.moveToFirst()) {
            do {
                Contact contact = new Contact();
                contact._id = Integer.parseInt(cursor.getString(0));
                contact._name = cursor.getString(1);
                contact._phoneNumber = cursor.getString(2);
                contactList.add(contact);
            } while (cursor.moveToNext());
        }
        return contactList;
    }

    /**
     * Get total row on database
     * @return int
     */
    public int getContactsCount(){
        String countQuery = "SELECT * FROM " + TABLE_CONTACTS;
        SQLiteDatabase db = this.getReadableDatabase();
        Cursor cursor = db.rawQuery(countQuery, null);
        cursor.close();
        return cursor.getCount();
    }

    /**
     * Updating contact
     * @param contact
     * @return int
     */
    public int updateContact(Contact contact){
        SQLiteDatabase db = this.getWritableDatabase();
        ContentValues values = new ContentValues();
        values.put(COLUMN_NAME, contact._name);
        values.put(COLUMN_PHONE, contact._phoneNumber);

        //updating row
        return db.update(TABLE_CONTACTS, values, COLUMN_ID + "=?",
                new String[]{String.valueOf(contact._id)});
    }

    /**
     * Deleting contact
     * @param contact
     */
    public void deleteContact(Contact contact){
        SQLiteDatabase db = this.getWritableDatabase();
        db.delete(TABLE_CONTACTS, COLUMN_ID + "=?",
                new String[]{String.valueOf(contact._id)});
        db.close();
    }

}
